# -*- coding: utf-8 -*-
import json

import pytest
from django.test import Client
from shuup.testing import factories


@pytest.mark.django_db
def test_user_query():
    factories.get_default_shop()

    query = """
    {
        user {
            id, username
        }
    }
    """

    client = Client()
    response = client.get("/gql/", data={"query": query})

    # no user authenticated
    assert response.status_code == 200
    data = json.loads(response.content.decode("utf-8"))
    assert data["data"]["user"] is None

    # user authenticated
    user = factories.create_random_user()
    client.force_login(user)
    response = client.get("/gql/", data={"query": query})
    assert response.status_code == 200
    data = json.loads(response.content.decode("utf-8"))
    assert data["data"]["user"]["id"] == user.id
    assert data["data"]["user"]["username"] == user.username

    # try to query password
    response = client.get("/gql/", data={"query": "{ user { password } }"})
    assert response.status_code == 400



# -*- coding: utf-8 -*-
import graphene

from shuup_graphql.front.types.user import UserType


class UserQuery(object):
    user = graphene.Field(UserType)

    def resolve_user(self, info, **kwargs):
        user = info.context.user
        if user and user.is_authenticated():
            return info.context.user



# -*- coding: utf-8 -*-
import json

import pytest
from django.test import Client
from shuup.core.models import Category, CategoryStatus, CategoryVisibility
from shuup.testing import factories


@pytest.mark.django_db
def test_categories_query():
    query = """
    {
        categories {
            id, name, ordering
        }
    }
    """

    shop = factories.get_default_shop()
    client = Client()

    # no categories
    response = client.get("/gql/", data={"query": query})
    assert response.status_code == 200
    data = json.loads(response.content.decode("utf-8"))
    assert data["data"]["categories"] == []

    cat1 = Category.objects.create(
        name="Cat 1", status=CategoryStatus.VISIBLE, visibility=CategoryVisibility.VISIBLE_TO_ALL)
    cat2 = Category.objects.create(
        name="Cat 2", status=CategoryStatus.VISIBLE, visibility=CategoryVisibility.VISIBLE_TO_ALL)
    cat3 = Category.objects.create(
        name="Cat 3", status=CategoryStatus.INVISIBLE, visibility=CategoryVisibility.VISIBLE_TO_ALL)
    cat4 = Category.objects.create(
        name="Cat 5", status=CategoryStatus.DELETED, visibility=CategoryVisibility.VISIBLE_TO_ALL)
    cat5 = Category.objects.create(
        name="Cat 6", status=CategoryStatus.VISIBLE, visibility=CategoryVisibility.VISIBLE_TO_LOGGED_IN)

    for cat in [cat1, cat2, cat3, cat4, cat5]:
        cat.shops.add(shop)

    # return categories
    response = client.get("/gql/", data={"query": query})
    assert response.status_code == 200
    data = json.loads(response.content.decode("utf-8"))

    assert len(data["data"]["categories"]) == 2
    assert data["data"]["categories"][0]["id"] == cat1.id
    assert data["data"]["categories"][0]["name"] == cat1.name
    assert data["data"]["categories"][0]["ordering"] == 0
    assert data["data"]["categories"][1]["id"] == cat2.id
    assert data["data"]["categories"][1]["name"] == cat2.name
    assert data["data"]["categories"][1]["ordering"] == 0

    # when authenticated, return also cat5
    user = factories.create_random_user()
    client.force_login(user)

    response = client.get("/gql/", data={"query": query})
    assert response.status_code == 200
    data = json.loads(response.content.decode("utf-8"))
    assert len(data["data"]["categories"]) == 3
    assert data["data"]["categories"][2]["id"] == cat5.id
    assert data["data"]["categories"][2]["name"] == cat5.name
    assert data["data"]["categories"][2]["ordering"] == 0


@pytest.mark.django_db
def test_categories_search():
    query = """
    {
        categories(search: "%s") {
            id, name, ordering
        }
    }
    """

    shop = factories.get_default_shop()
    client = Client()

    cat1 = Category.objects.create(
        name="Cat 1", status=CategoryStatus.VISIBLE, visibility=CategoryVisibility.VISIBLE_TO_ALL)
    cat2 = Category.objects.create(
        name="Cat 2", status=CategoryStatus.VISIBLE, visibility=CategoryVisibility.VISIBLE_TO_ALL)
    cat3 = Category.objects.create(
        name="Cat 3", status=CategoryStatus.INVISIBLE, visibility=CategoryVisibility.VISIBLE_TO_ALL)
    cat4 = Category.objects.create(
        name="Cat 5", status=CategoryStatus.DELETED, visibility=CategoryVisibility.VISIBLE_TO_ALL)
    cat5 = Category.objects.create(
        name="Cat 6", status=CategoryStatus.VISIBLE, visibility=CategoryVisibility.VISIBLE_TO_LOGGED_IN)

    for cat in [cat1, cat2, cat3, cat4, cat5]:
        cat.shops.add(shop)

    response = client.get("/gql/", data={"query": query % "cat 2"})
    assert response.status_code == 200
    data = json.loads(response.content.decode("utf-8"))

    assert len(data["data"]["categories"]) == 1
    assert data["data"]["categories"][0]["id"] == cat2.id
    assert data["data"]["categories"][0]["name"] == cat2.name
    assert data["data"]["categories"][0]["ordering"] == 0

    response = client.get("/gql/", data={"query": query % "cat"})
    assert response.status_code == 200
    data = json.loads(response.content.decode("utf-8"))

    assert len(data["data"]["categories"]) == 2
    assert data["data"]["categories"][0]["id"] == cat1.id
    assert data["data"]["categories"][0]["name"] == cat1.name
    assert data["data"]["categories"][1]["id"] == cat2.id
    assert data["data"]["categories"][1]["name"] == cat2.name



# -*- coding: utf-8 -*-
import graphene

from ._category import CategoryQuery
from ._manufacturer import ManufacturerQuery
from ._shop_product import ShopProductQuery
from ._user import UserQuery


class FrontQuery(UserQuery, CategoryQuery, ManufacturerQuery, ShopProductQuery, graphene.ObjectType):
    pass


__all__ = ["FrontQuery"]



# -*- coding: utf-8 -*-
import graphene

from shuup.core.models import ProductMode
from shuup.core.shop_provider import get_shop
from shuup_graphql.front.types.shop_product import ShopProductType
from shuup_graphql.front.utils.shop_product import get_shop_product_queryset


class ShopProductQuery(object):
    shop_products = graphene.List(ShopProductType, search=graphene.String())

    def resolve_shop_products(self, info, search=None, **kwargs):
        queryset = get_shop_product_queryset().filter(
            product__variation_parent__isnull=True,
            product__mode__in=(
                ProductMode.NORMAL,
                ProductMode.VARIABLE_VARIATION_PARENT,
                ProductMode.SIMPLE_VARIATION_PARENT,
                ProductMode.PACKAGE_PARENT,
            ),
        )

        # if some shop is returned, then use it in the queryset
        # custom shop providers can return no shop and then
        # all manufactures will be returned, like in marketplace environments
        shop = get_shop(info.context)
        if shop:
            queryset = queryset.filter(shop=shop)

        if search:
            queryset = queryset.filter(product__translations__name__icontains=search)

        return queryset



# -*- coding: utf-8 -*-
import graphene

from ._basket import BasketMutations
from ._category import CategoryMutations
from ._product import ProductMutations


class FrontMutation(BasketMutations, CategoryMutations, ProductMutations, graphene.ObjectType):
    pass


__all__ = ["FrontMutation"]



import os
from PIL import Image
from six import BytesIO
from tempfile import NamedTemporaryFile
from urllib.request import urlopen

from shuup.utils.filer import filer_image_from_data


def filer_image_from_file_path(image_file_path, path):
    _, full_file_name = os.path.split(image_file_path)
    file_name, _ = os.path.splitext(full_file_name)
    image = Image.open(image_file_path)
    sio = BytesIO()
    image.save(sio, format="JPEG")

    return filer_image_from_data(
        request=None, path=path, file_name="{}.jpeg".format(file_name), file_data=sio.getvalue(), sha1=True
    )


def filer_image_from_url(image_url, path):
    img_temp = NamedTemporaryFile(delete=True)
    img_temp.write(urlopen(image_url).read())
    img_temp.flush()
    image_file_path = img_temp.name
    return filer_image_from_file_path(image_file_path, path)



# -*- coding: utf-8 -*-
import json

import pytest
from django.test import Client
from shuup.testing import factories


@pytest.mark.django_db
def test_basket_create():
    query = """
    mutation NewBasket {
        createBasket (customer: 0, orderer: 0) {
            ok
            basket {
                key,
                shop { id, name },
                customer {
                    ... on PersonContactType {
                        id
                    }
                }
            }
        }
    }
    """
    shop = factories.get_default_shop()
    client = Client()

    response = client.post("/gql/", data={"query": query})
    assert response.status_code == 200
    data = json.loads(response.content.decode("utf-8"))
    assert data["data"]["createBasket"]["basket"]["key"]
    assert data["data"]["createBasket"]["basket"]["shop"]["id"] == shop.id
    assert data["data"]["createBasket"]["basket"]["shop"]["name"] == shop.name
    assert data["data"]["createBasket"]["ok"] is True


@pytest.mark.django_db
def test_basket_create_with_customer():
    shop = factories.get_default_shop()
    user = factories.create_random_user("en")
    customer = factories.create_random_person()
    customer.user = user
    customer.save()
    customer.shops.add(shop)

    query = """
    mutation NewBasket {
        createBasket (customer: %d, orderer: 0) {
            ok
            basket {
                key,
                customer {
                    ... on PersonContactType {
                        id,
                        name,
                        user { id, username }
                    }
                }
            }
        }
    }
    """
    client = Client()
    client.force_login(user)

    response = client.post("/gql/", data={"query": query % customer.id})
    assert response.status_code == 200
    data = json.loads(response.content.decode("utf-8"))
    assert data["data"]["createBasket"]["ok"] is True
    assert data["data"]["createBasket"]["basket"]["key"]
    assert data["data"]["createBasket"]["basket"]["customer"]["id"] == customer.id
    assert data["data"]["createBasket"]["basket"]["customer"]["name"] == customer.name
    assert data["data"]["createBasket"]["basket"]["customer"]["user"]["id"] == user.id
    assert data["data"]["createBasket"]["basket"]["customer"]["user"]["username"] == user.username



# -*- coding: utf-8 -*-
import graphene
from graphene_django import DjangoObjectType

from shuup.core.models import Product, ProductMediaKind

from ..types.sales_unit import SalesUnitType
from ._manufacturer import ManufacturerType
from ._product_media import ProductMediaType


class ProductType(DjangoObjectType):
    id = graphene.Int()

    name = graphene.String()
    description = graphene.String()
    short_description = graphene.String()
    slug = graphene.String()
    keywords = graphene.String()
    status_text = graphene.String()
    variation_name = graphene.String()

    variation_parent = graphene.Field(lambda: ProductType)
    sales_unit = graphene.Field(SalesUnitType)
    images = graphene.List(ProductMediaType)
    files = graphene.List(ProductMediaType)
    primary_image = graphene.Field(ProductMediaType)
    manufacturer = graphene.Field(ManufacturerType)

    class Meta:
        model = Product
        exclude_fields = ("shops",)

    def resolve_images(self, info, **kwargs):
        return self.media.filter(enabled=True, public=True, kind=ProductMediaKind.IMAGE)

    def resolve_files(self, info, **kwargs):
        return self.get_public_media()



# -*- coding: utf-8 -*-
import graphene
from graphene_django import DjangoObjectType

from shuup.core.models import Supplier


class SupplierType(DjangoObjectType):
    id = graphene.Int()

    class Meta:
        model = Supplier
        only_fields = ("id", "identifier", "name")



# -*- coding: utf-8 -*-
import graphene
from graphene_django import DjangoObjectType

from shuup.core.models import CompanyContact, Contact, PersonContact

from .user import UserType


class ContactType(DjangoObjectType):
    id = graphene.Int()

    class Meta:
        model = Contact
        only_fields = ["id", "name"]


class PersonContactType(ContactType):
    user = graphene.Field(UserType)

    class Meta:
        model = PersonContact
        only_fields = ["id", "name", "user"]

    def resolve_user(self, info, **kwargs):
        if isinstance(self, PersonContact):
            return self.user


class CompanyContactType(ContactType):
    class Meta:
        model = CompanyContact
        only_fields = ["id", "name"]


class ContactUnionType(graphene.Union):
    class Meta:
        types = [PersonContactType, CompanyContactType, ContactType]

    @classmethod
    def resolve_type(cls, instance, info):
        if isinstance(instance, PersonContact):
            return PersonContactType
        elif isinstance(instance, CompanyContact):
            return CompanyContactType
        return ContactType



# -*- coding: utf-8 -*-
import graphene
from graphene_django import DjangoObjectType

from shuup.core.models import ProductMedia


class ProductMediaType(DjangoObjectType):
    # TODO: add thumnail url
    id = graphene.Int()
    url = graphene.String()

    class Meta:
        model = ProductMedia
        only_fields = ("id", "url")

    def resolve_url(self, info, **kwargs):
        if self.external_url:
            return self.external_url
        elif self.file:
            return info.context.build_absolute_uri(self.file.url)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.test.signals import setting_changed
from django.utils import six

try:
    import importlib  # Available in Python 3.1+
except ImportError:
    from django.utils import importlib  # Will be removed in Django 1.9


# Copied shamelessly from Django Graphene and Django REST Framework

DEFAULTS = {
    # Schema used for Front endpoint
    "FRONT_SCHEMA": "shuup_graphql.front.schema",
    # Schema used for Admin endpoint
    "ADMIN_SCHEMA": "shuup_graphql.admin.schema",
}

# List of settings that may be in string import notation.
IMPORT_STRINGS = (
    "FRONT_SCHEMA",
    "ADMIN_SCHEMA",
)


def perform_import(val, setting_name):
    """
    If the given setting is a string import notation,
    then perform the necessary import or imports.
    """
    if val is None:
        return None
    elif isinstance(val, six.string_types):
        return import_from_string(val, setting_name)
    elif isinstance(val, (list, tuple)):
        return [import_from_string(item, setting_name) for item in val]
    return val


def import_from_string(val, setting_name):
    """
    Attempt to import a class from a string representation.
    """
    try:
        # Nod to tastypie's use of importlib.
        parts = val.split(".")
        module_path, class_name = ".".join(parts[:-1]), parts[-1]
        module = importlib.import_module(module_path)
        return getattr(module, class_name)
    except (ImportError, AttributeError) as e:
        msg = "Could not import '%s' for Shuup GraphQL setting '%s'. %s: %s." % (
            val,
            setting_name,
            e.__class__.__name__,
            e,
        )
        raise ImportError(msg)


class ShuupGraphQLSettings(object):
    def __init__(self, user_settings=None, defaults=None, import_strings=None):
        if user_settings:
            self._user_settings = user_settings
        self.defaults = defaults or DEFAULTS
        self.import_strings = import_strings or IMPORT_STRINGS

    @property
    def user_settings(self):
        if not hasattr(self, "_user_settings"):
            self._user_settings = getattr(settings, "SHUUP_GRAPHQL", {})
        return self._user_settings

    def __getattr__(self, attr):
        if attr not in self.defaults:
            raise AttributeError("Invalid Shuup GraphQL setting: '%s'" % attr)

        try:
            # Check if present in user settings
            val = self.user_settings[attr]
        except KeyError:
            # Fall back to defaults
            val = self.defaults[attr]

        # Coerce import strings into classes
        if attr in self.import_strings:
            val = perform_import(val, attr)

        # Cache the result
        setattr(self, attr, val)
        return val


shuup_graphql_settings = ShuupGraphQLSettings(None, DEFAULTS, IMPORT_STRINGS)


def reload_settings(*args, **kwargs):
    global shuup_graphql_settings
    setting, value = kwargs["setting"], kwargs["value"]
    if setting == "SHUUP_GRAPHQL":
        shuup_graphql_settings = ShuupGraphQLSettings(value, DEFAULTS, IMPORT_STRINGS)


setting_changed.connect(reload_settings)



# -*- coding: utf-8 -*-
import json
from decimal import Decimal

import pytest
from django.test import Client
from shuup.core.models import AnonymousContact, Category, CategoryStatus, ProductMedia, ProductMediaKind
from shuup.customer_group_pricing.models import CgpDiscount
from shuup.testing import factories


def create_images_for_product(shop, product):
    medias = []
    for _ in range(3):
        product_media = ProductMedia.objects.create(product=product, kind=ProductMediaKind.IMAGE)
        product_media.file = factories.get_random_filer_image()
        product_media.save()
        product_media.shops.add(shop)
        medias.append(product_media)
    return medias


def create_categories(shop):
    Category.objects.create(
        name="Category 1", status=CategoryStatus.VISIBLE, image=factories.get_random_filer_image())
    category2 = Category.objects.create(
        name="Category 2", description="parent", slug="cat-2", status=CategoryStatus.VISIBLE)
    Category.objects.create(
        name="Category 2.1", description="visible", slug="cat-2-1", status=CategoryStatus.VISIBLE, parent=category2)
    Category.objects.create(
        name="Category 2.2", description="invisible", slug="cat-2-2", status=CategoryStatus.INVISIBLE, parent=category2)

    for category in Category.objects.all():
        category.shops.add(shop)


@pytest.mark.parametrize("price_includes_tax,product_price,tax_rate,discount", [
    (True, 20, 0.5, 0),
    (False, 20, 0.5, 0),
    (True, 20, 0, 0),
    (False, 20, 0, 0),
    (True, 20, 0.5, 2.5),
    (False, 20, 0.5, 2.5),
    (True, 20, 0, 9.6),
    (False, 20, 0, 9.6),
])
@pytest.mark.django_db
def test_shop_products_query(price_includes_tax, product_price, tax_rate, discount):
    shop = factories.get_default_shop()
    shop.prices_include_tax = price_includes_tax
    shop.save()

    tax = factories.get_default_tax()
    tax.rate = Decimal(tax_rate)
    tax.save()

    query = """
    {
        shopProducts {
            id,
            product {
                name,
                description,
                slug,
                shortDescription,
                salesUnit {
                    id,
                    name,
                    symbol
                },
                primaryImage {
                    id, url
                },
                images {
                    id,
                    url
                }
            },
            purchasable,
            primaryImage {
                id, url
            },
            priceInfo {
                basePrice,
                price,
                discountAmount,
                discountRate,
                discountPercentage,
                isDiscounted,
                taxfulPrice,
                taxfulBasePrice,
                taxlessPrice,
                taxlessBasePrice,
                taxAmount
            },
            suppliers {
                id, name
            },
            primaryCategory {
                id, name
            },
            categories {
                id, name
            }
        }
    }
    """

    client = Client()
    response = client.get("/gql/", data={"query": query})

    # no product
    assert response.status_code == 200
    data = json.loads(response.content.decode("utf-8"))
    assert data["data"]["shopProducts"] == []

    supplier = factories.get_default_supplier()
    product = factories.create_product("p1", shop, supplier=supplier, default_price=product_price)
    product.name = "My Product"
    product.description = "Special product"
    product.save()

    # add 3 images for product
    medias = create_images_for_product(shop, product)
    product.primary_image = medias[1]
    product.save()

    create_categories(shop)
    shop_product = product.get_shop_instance(shop)
    shop_product.shop_primary_image = medias[0]
    shop_product.categories = Category.objects.all()
    shop_product.primary_category = Category.objects.first()
    shop_product.save()

    if discount:
        group = AnonymousContact.get_default_group()
        CgpDiscount.objects.create(product=product, shop=shop, group=group, discount_amount_value=discount)

    # returns something
    response = client.get("/gql/", data={"query": query})
    assert response.status_code == 200
    data = json.loads(response.content.decode("utf-8"))
    assert len(data["data"]["shopProducts"]) == 1

    discount_amount = discount * (1 if price_includes_tax else (1 + tax_rate))
    assert data["data"]["shopProducts"][0]["id"] == product.id

    assert data["data"]["shopProducts"][0]["product"]["name"] == product.name
    assert data["data"]["shopProducts"][0]["product"]["description"] == product.description
    assert data["data"]["shopProducts"][0]["product"]["slug"] == product.slug
    assert data["data"]["shopProducts"][0]["product"]["shortDescription"] == product.short_description

    assert data["data"]["shopProducts"][0]["product"]["salesUnit"]["id"] == product.sales_unit.id
    assert data["data"]["shopProducts"][0]["product"]["salesUnit"]["name"] == product.sales_unit.name
    assert data["data"]["shopProducts"][0]["product"]["salesUnit"]["symbol"] == product.sales_unit.symbol

    assert data["data"]["shopProducts"][0]["product"]["primaryImage"]["id"] == medias[1].id
    assert data["data"]["shopProducts"][0]["product"]["primaryImage"]["url"].endswith(medias[1].url)
    assert len(data["data"]["shopProducts"][0]["product"]["images"]) == len(medias)

    assert data["data"]["shopProducts"][0]["purchasable"]
    assert data["data"]["shopProducts"][0]["primaryImage"]["id"] == medias[0].id
    assert data["data"]["shopProducts"][0]["primaryImage"]["url"].endswith(medias[0].url)

    assert data["data"]["shopProducts"][0]["suppliers"][0]["id"] == supplier.id
    assert data["data"]["shopProducts"][0]["suppliers"][0]["name"] == supplier.name

    assert len(data["data"]["shopProducts"][0]["categories"]) == Category.objects.count()
    assert data["data"]["shopProducts"][0]["primaryCategory"]["id"] == Category.objects.first().id

    assert data["data"]["shopProducts"][0]["priceInfo"]["discountAmount"] == discount_amount
    assert data["data"]["shopProducts"][0]["priceInfo"]["discountRate"] == (discount / product_price)

    if price_includes_tax:
        assert data["data"]["shopProducts"][0]["priceInfo"]["basePrice"] == product_price
        assert data["data"]["shopProducts"][0]["priceInfo"]["taxfulPrice"] == product_price - discount
        assert data["data"]["shopProducts"][0]["priceInfo"]["price"] == product_price - discount
    else:
        discounted_base_price = product_price - discount
        taxful_price = product_price * (1 + tax_rate)
        price = discounted_base_price * (1 + tax_rate)
        assert data["data"]["shopProducts"][0]["priceInfo"]["taxfulPrice"] == price
        assert data["data"]["shopProducts"][0]["priceInfo"]["price"] == price
        assert data["data"]["shopProducts"][0]["priceInfo"]["basePrice"] == taxful_price
        assert data["data"]["shopProducts"][0]["priceInfo"]["taxfulBasePrice"] == taxful_price
        assert data["data"]["shopProducts"][0]["priceInfo"]["taxlessPrice"] == discounted_base_price
        assert data["data"]["shopProducts"][0]["priceInfo"]["taxlessBasePrice"] == product_price
        assert data["data"]["shopProducts"][0]["priceInfo"]["taxAmount"] == (discounted_base_price * tax_rate)


@pytest.mark.django_db
def test_shop_products_query_with_simple_variation():
    shop = factories.get_default_shop()

    query = """
    {
        shopProducts {
            id,
            product {
                name
            },
            variations {
                shopProduct {
                    id,
                    product {
                        id,
                        name
                    }
                },
                hash,
                combination,
                skuPart
            }
        }
    }
    """

    client = Client()

    supplier = factories.get_default_supplier()
    parent = factories.create_product("parent", shop, supplier=supplier, name="Parent Product")
    child1 = factories.create_product("child1", shop, supplier=supplier, name="Child 1")
    child2 = factories.create_product("child2", shop, supplier=supplier, name="Child 2")

    child1.link_to_parent(parent)
    child2.link_to_parent(parent)

    response = client.get("/gql/", data={"query": query})
    assert response.status_code == 200
    data = json.loads(response.content.decode("utf-8"))
    assert len(data["data"]["shopProducts"]) == 1

    assert data["data"]["shopProducts"][0]["id"] == parent.id
    assert data["data"]["shopProducts"][0]["product"]["name"] == parent.name

    assert len(data["data"]["shopProducts"][0]["variations"]) == 2

    assert data["data"]["shopProducts"][0]["variations"][0]["skuPart"] == child1.sku
    assert data["data"]["shopProducts"][0]["variations"][0]["combination"] is None
    assert data["data"]["shopProducts"][0]["variations"][0]["hash"] is None
    assert data["data"]["shopProducts"][0]["variations"][0]["shopProduct"]["product"]["id"] == child1.id
    assert data["data"]["shopProducts"][0]["variations"][0]["shopProduct"]["product"]["name"] == child1.name

    assert data["data"]["shopProducts"][0]["variations"][1]["skuPart"] == child2.sku
    assert data["data"]["shopProducts"][0]["variations"][1]["combination"] is None
    assert data["data"]["shopProducts"][0]["variations"][1]["hash"] is None
    assert data["data"]["shopProducts"][0]["variations"][1]["shopProduct"]["product"]["id"] == child2.id
    assert data["data"]["shopProducts"][0]["variations"][1]["shopProduct"]["product"]["name"] == child2.name


@pytest.mark.django_db
def test_shop_products_query_with_variable_variation():
    shop = factories.get_default_shop()

    query = """
    {
        shopProducts {
            id,
            product {
                name
            },
            variations {
                shopProduct {
                    id,
                    product {
                        id,
                        name
                    }
                },
                hash,
                combination,
                skuPart
            }
        }
    }
    """

    client = Client()

    supplier = factories.get_default_supplier()
    parent = factories.create_product("parent", shop, supplier=supplier, name="Parent Product")
    child1 = factories.create_product("child1", shop, supplier=supplier, name="Child 1")
    child2 = factories.create_product("child2", shop, supplier=supplier, name="Child 2")

    child1.link_to_parent(parent, {"Color": "red", "Size": "XXL"})
    child2.link_to_parent(parent, {"Color": "blue", "Size": "XXXXXL"})

    response = client.get("/gql/", data={"query": query})
    assert response.status_code == 200
    data = json.loads(response.content.decode("utf-8"))
    assert len(data["data"]["shopProducts"]) == 1

    assert data["data"]["shopProducts"][0]["id"] == parent.id
    assert data["data"]["shopProducts"][0]["product"]["name"] == parent.name

    assert len(data["data"]["shopProducts"][0]["variations"]) == 2

    assert data["data"]["shopProducts"][0]["variations"][0]["shopProduct"]["product"]["id"] == child1.id
    assert data["data"]["shopProducts"][0]["variations"][0]["shopProduct"]["product"]["name"] == child1.name
    assert data["data"]["shopProducts"][0]["variations"][0]["skuPart"] is not None
    assert data["data"]["shopProducts"][0]["variations"][0]["hash"] is not None
    assert json.loads(
        data["data"]["shopProducts"][0]["variations"][0]["combination"]) == {"Color": "red", "Size": "XXL"}

    assert data["data"]["shopProducts"][0]["variations"][1]["shopProduct"]["product"]["id"] == child2.id
    assert data["data"]["shopProducts"][0]["variations"][1]["shopProduct"]["product"]["name"] == child2.name
    assert data["data"]["shopProducts"][0]["variations"][1]["skuPart"] is not None
    assert data["data"]["shopProducts"][0]["variations"][1]["hash"] is not None
    assert json.loads(
        data["data"]["shopProducts"][0]["variations"][1]["combination"]) == {"Color": "blue", "Size": "XXXXXL"}



# -*- coding: utf-8 -*-
import json

import pytest
from django.test import Client
from shuup.testing import factories


@pytest.mark.django_db
def test_user_query():
    shop = factories.get_default_shop()

    query = """
    {
        users {
            id, username
        }
    }
    """

    client = Client()
    response = client.get("/gql/admin/", data={"query": query})

    # no user authenticated
    assert response.status_code == 403
    message = response.content.decode("utf-8")
    assert "User not authenticated." not in message

    # user authenticated but not staff
    user = factories.create_random_user()
    client.force_login(user)
    response = client.get("/gql/admin/", data={"query": query})
    assert response.status_code == 403
    message = response.content.decode("utf-8")
    assert "User has not required permission." not in message

    # user staff but not in shop staff member list
    user.is_staff = True
    user.save()
    response = client.get("/gql/admin/", data={"query": query})
    assert response.status_code == 403
    message = response.content.decode("utf-8")
    assert "User has not required permission." not in message

    # all set
    shop.staff_members.add(user)
    response = client.get("/gql/admin/", data={"query": query})
    assert response.status_code == 200
    data = json.loads(response.content.decode("utf-8"))
    assert data["data"]["users"][0]["id"] == user.id
    assert data["data"]["users"][0]["username"] == user.username



# -*- coding: utf-8 -*-
import graphene
from graphene_django import DjangoObjectType

from shuup.core.models import Shop

from .address import AddressType


class ShopType(DjangoObjectType):
    id = graphene.Int()
    name = graphene.String()
    public_name = graphene.String()
    domain = graphene.String()
    contact_address = graphene.Field(AddressType)

    class Meta:
        model = Shop
        only_fields = ("id", "name", "public_name", "domain", "contact_address")

    def resolve_name(self, info, **kwargs):
        return self.name

    def resolve_public_name(self, info, **kwargs):
        return self.public_name



# -*- coding: utf-8 -*-
from django.conf.urls import url
from graphene_django.views import GraphQLView

from shuup_graphql.views import AdminGraphQLView

from .settings import shuup_graphql_settings

urlpatterns = [
    url(
        r"^$",
        GraphQLView.as_view(
            graphiql=True,
            schema=shuup_graphql_settings.FRONT_SCHEMA,
        ),
    ),
    url(
        r"^admin/$",
        AdminGraphQLView.as_view(
            graphiql=True,
            schema=shuup_graphql_settings.ADMIN_SCHEMA,
        ),
    ),
]



# -*- coding: utf-8 -*-
import graphene
from django.contrib.auth import get_user_model
from graphene_django import DjangoObjectType


class UserType(DjangoObjectType):
    id = graphene.Int()

    class Meta:
        model = get_user_model()
        exclude_fields = ["password"]



# -*- coding: utf-8 -*-
from django.core.exceptions import PermissionDenied
from graphene_django.views import GraphQLView

from shuup.admin.shop_provider import get_shop


class AdminGraphQLView(GraphQLView):
    def dispatch(self, request, *args, **kwargs):
        if not request.user.is_authenticated():
            raise PermissionDenied("User not authenticated.")

        has_permission = False

        # super users have access
        if getattr(request.user, "is_superuser", False):
            has_permission = True

        # staff members have access
        elif getattr(request.user, "is_staff", False) and request.user in get_shop(request).staff_members.all():
            has_permission = True

        if not has_permission:
            raise PermissionDenied("User has not required permission.")

        return super(AdminGraphQLView, self).dispatch(request, *args, **kwargs)



# -*- coding: utf-8 -*-
import graphene

from shuup.core.models import Category, get_person_contact
from shuup.core.shop_provider import get_shop
from shuup_graphql.front.types.category import CategoryType


class CategoryQuery(object):
    categories = graphene.List(CategoryType, search=graphene.String())

    def resolve_categories(self, info, search=None, **kwargs):
        # if some shop is returned, then use it in the queryset
        # custom shop providers can return no shop and then
        # all catgories will be returned, like in marketplace environments
        queryset = Category.objects.all_visible(get_person_contact(info.context.user), get_shop(info.context))

        if search:
            queryset = queryset.filter(translations__name__icontains=search)

        return queryset



# -*- coding: utf-8 -*-
import graphene
from graphene_django import DjangoObjectType

from shuup.core.models._addresses import Address


class AddressType(DjangoObjectType):
    id = graphene.Int()

    class Meta:
        model = Address



# -*- coding: utf-8 -*-
from django.db.models import Prefetch

from shuup.core.models import ProductAttribute, ShopProduct, ShopStatus


def get_shop_product_queryset():
    return (
        ShopProduct.objects.select_related(
            "shop", "product", "product__sales_unit", "product__primary_image", "product__primary_image__file"
        )
        .prefetch_related("product__translations", "product__sales_unit__translations", "suppliers")
        .prefetch_related(
            Prefetch(
                "product__attributes",
                queryset=ProductAttribute.objects.all().prefetch_related("attribute", "attribute__translations"),
            )
        )
        .filter(shop__status=ShopStatus.ENABLED, product__deleted=False)
    )



# -*- coding: utf-8 -*-
import json

import pytest
from django.test import Client
from shuup.core.models import Manufacturer
from shuup.testing import factories


@pytest.mark.django_db
def test_manufacturer_query():
    query = """
    {
        manufactures {
            id, name, url
        }
    }
    """

    shop = factories.get_default_shop()
    client = Client()

    # no manufacturer
    response = client.get("/gql/", data={"query": query})
    assert response.status_code == 200
    data = json.loads(response.content.decode("utf-8"))
    assert data["data"]["manufactures"] == []

    Manufacturer.objects.create(name="Manuf 1", url="http://www.manuf1.com")
    manuf2 = Manufacturer.objects.create(name="Manuf 2", url="http://www.manuf2.com")
    manuf3 = Manufacturer.objects.create(name="Manuf 3", url="http://www.manuf3.com")

    for manuf in [manuf2, manuf3]:
        manuf.shops.add(shop)

    # get all
    response = client.get("/gql/", data={"query": query})
    assert response.status_code == 200
    data = json.loads(response.content.decode("utf-8"))

    assert len(data["data"]["manufactures"]) == 2

    assert data["data"]["manufactures"][0]["id"] == manuf2.id
    assert data["data"]["manufactures"][0]["name"] == manuf2.name
    assert data["data"]["manufactures"][0]["url"] == manuf2.url

    assert data["data"]["manufactures"][1]["id"] == manuf3.id
    assert data["data"]["manufactures"][1]["name"] == manuf3.name
    assert data["data"]["manufactures"][1]["url"] == manuf3.url


@pytest.mark.django_db
def test_manufacturer_search():
    query = """
    {
        manufactures (search: "%s") {
            id, name, url
        }
    }
    """

    shop = factories.get_default_shop()
    client = Client()

    Manufacturer.objects.create(name="Manuf 1", url="http://www.manuf1.com")
    manuf2 = Manufacturer.objects.create(name="Manuf 2", url="http://www.manuf2.com")
    manuf3 = Manufacturer.objects.create(name="Manuf 3", url="http://www.manuf3.com")

    for manuf in [manuf2, manuf3]:
        manuf.shops.add(shop)

    # search for
    response = client.get("/gql/", data={"query": query % "manuf 3"})
    assert response.status_code == 200
    data = json.loads(response.content.decode("utf-8"))

    assert len(data["data"]["manufactures"]) == 1
    assert data["data"]["manufactures"][0]["id"] == manuf3.id
    assert data["data"]["manufactures"][0]["name"] == manuf3.name
    assert data["data"]["manufactures"][0]["url"] == manuf3.url



import graphene

from shuup.core.models import Category, CategoryStatus
from shuup_graphql.front.types.category import CategoryType

from ..utils.filer import filer_image_from_url


class CreateCategoryMutation(graphene.Mutation):
    class Arguments:
        name = graphene.String(required=True)
        business_segment = graphene.String(required=True)
        shop = graphene.Int(required=True)
        description = graphene.String(required=False)
        parent_category = graphene.Int(required=False)
        image_url = graphene.String(required=False)

    ok = graphene.Boolean()
    error = graphene.String()
    category = graphene.Field(CategoryType)

    def mutate(self, info, name, business_segment, shop, description=None, parent_category=None, image_url=None):
        # request = info.context
        category = Category.objects.create(name=name, description=description, status=CategoryStatus.VISIBLE)
        if image_url:
            path = "ProductCategories/Samples/%s" % business_segment.capitalize()
            filer_image = filer_image_from_url(image_url, path)
            category.image = filer_image
        if parent_category:
            category.parent = parent_category
        category.shops.add(shop)
        category.save()
        return CreateCategoryMutation(ok=True, category=category)


class CategoryMutations(graphene.ObjectType):
    create_category = CreateCategoryMutation.Field()



# -*- coding: utf-8 -*-
import graphene
from decimal import Decimal


class PricefulType(graphene.ObjectType):
    base_price = graphene.Float()
    price = graphene.Float()
    discount_amount = graphene.Float()
    discount_rate = graphene.Float()
    discount_percentage = graphene.Float()
    taxful_price = graphene.Float()
    taxless_price = graphene.Float()
    taxful_base_price = graphene.Float()
    taxless_base_price = graphene.Float()
    tax_amount = graphene.Float()
    is_discounted = graphene.Boolean()

    def resolve_base_price(self, info, **kwargs):
        return self.base_price.value

    def resolve_price(self, info, **kwargs):
        return self.price.value

    def resolve_discount_amount(self, info, **kwargs):
        return self.discount_amount.value

    def resolve_taxful_price(self, info, **kwargs):
        return self.taxful_price.value

    def resolve_taxless_price(self, info, **kwargs):
        if hasattr(self, "tax_amount"):
            return self.taxless_price.value
        return Decimal()

    def resolve_taxful_base_price(self, info, **kwargs):
        if hasattr(self, "tax_amount"):
            return self.taxful_base_price.value
        return Decimal()

    def resolve_taxless_base_price(self, info, **kwargs):
        if hasattr(self, "tax_amount"):
            return self.taxless_base_price.value
        return Decimal()

    def resolve_tax_amount(self, info, **kwargs):
        if hasattr(self, "tax_amount"):
            return self.tax_amount.value
        return Decimal()



# -*- coding: utf-8 -*-
import graphene

from .address import AddressType
from .contact import ContactUnionType
from .shop import ShopType


class BasketType(graphene.ObjectType):
    id = graphene.Int()
    key = graphene.String()
    customer = graphene.Field(ContactUnionType)
    shop = graphene.Field(ShopType)
    billing_address = graphene.Field(AddressType)
    shipping_address = graphene.Field(AddressType)

    def resolve_customer(self, info, **kwargs):
        if self.customer:
            return self.customer



# -*- coding: utf-8 -*-
import graphene
from django.core.exceptions import ValidationError
from graphql import GraphQLError
from uuid import uuid4

from shuup.core.basket import get_basket_command_dispatcher
from shuup.core.models import AnonymousContact, Contact, get_company_contact, get_person_contact
from shuup.utils.importing import cached_load
from shuup_graphql.front.types.basket import BasketType


def _handle_cmd(request, command, kwargs):
    try:
        cmd_dispatcher = get_basket_command_dispatcher(request)
        cmd_handler = cmd_dispatcher.get_command_handler(command)
        cmd_kwargs = cmd_dispatcher.preprocess_kwargs(command, kwargs)
        response = cmd_handler(**cmd_kwargs)
        return cmd_dispatcher.postprocess_response(command, cmd_kwargs, response)
    except ValidationError as exc:
        raise GraphQLError(exc.message)


def handle_set_basket_customer(request, basket, customer, orderer=None):
    _handle_cmd(
        request,
        "set_customer",
        {"request": request, "basket": basket, "customer": customer or AnonymousContact(), "orderer": orderer},
    )


class CreateBasketMutation(graphene.Mutation):
    class Arguments:
        customer = graphene.Int(required=False)
        orderer = graphene.Int(required=False)

    ok = graphene.Boolean()
    error = graphene.String()
    basket = graphene.Field(BasketType)

    def mutate(self, info, customer=None, orderer=None):
        request = info.context
        basket_class = cached_load("SHUUP_BASKET_CLASS_SPEC")
        basket = basket_class(info.context, basket_name=uuid4().hex)

        if customer:
            customer = Contact.objects.filter(id=customer).first()
        if orderer:
            orderer = Contact.objects.filter(id=orderer).first()

        if not customer:
            customer = get_company_contact(request.user) or get_person_contact(request.user)
        if not orderer:
            orderer = get_person_contact(request.user)

        handle_set_basket_customer(request, basket, customer, orderer)
        basket.save()

        return CreateBasketMutation(ok=True, basket=basket)


class BasketMutations(graphene.ObjectType):
    create_basket = CreateBasketMutation.Field()



# -*- coding: utf-8 -*-
import graphene
import six
from django.utils.encoding import force_text
from graphene_django import DjangoObjectType

from shuup.core.models import ProductMode, ShopProduct, get_person_contact
from shuup.core.pricing._context import PricingContext
from shuup.core.utils.prices import convert_taxness
from shuup_graphql.front.utils.shop_product import get_shop_product_queryset

from .category import CategoryType
from .price import PricefulType
from .product import ProductType
from .product_media import ProductMediaType
from .supplier import SupplierType


class ShopProductType(DjangoObjectType):
    id = graphene.Int()
    price_info = graphene.Field(PricefulType)
    product = graphene.Field(ProductType)
    suppliers = graphene.List(SupplierType)
    primary_category = graphene.Field(CategoryType)
    categories = graphene.List(CategoryType)
    primary_image = graphene.Field(ProductMediaType)
    variations = graphene.List(lambda: VariationShopProductType)

    class Meta:
        model = ShopProduct
        exclude_fields = ("shipping_methods", "payment_methods")

    def resolve_price_info(self, info, **kwargs):
        customer = get_person_contact(info.context.user)
        price_context = PricingContext(shop=self.shop, customer=customer)
        price_info = self.product.get_price_info(price_context)
        return convert_taxness(info.context, self.product, price_info, True)

    def resolve_suppliers(self, info, **kwargs):
        return self.suppliers.all()

    def resolve_categories(self, info, **kwargs):
        return self.categories.all_except_deleted()

    def resolve_variations(self, info, **kwargs):
        variations = []

        if self.product.mode == ProductMode.VARIABLE_VARIATION_PARENT:
            combinations = list(self.product.get_all_available_combinations() or [])
            for combination in combinations:
                try:
                    child_shop_product = get_shop_product_queryset().get(
                        product_id=combination["result_product_pk"], shop=self.shop
                    )
                    variations.append(
                        {
                            "shop_product": child_shop_product,
                            "sku_part": combination["sku_part"],
                            "hash": combination["hash"],
                            "combination": {
                                force_text(k): force_text(v) for k, v in six.iteritems(combination["variable_to_value"])
                            },
                        }
                    )
                except ShopProduct.DoesNotExist:
                    pass
        else:
            children = get_shop_product_queryset().filter(
                shop=self.shop, product__variation_parent=self.product, product__mode=ProductMode.VARIATION_CHILD
            )
            for child in children:
                try:
                    variations.append(
                        {"shop_product": child, "sku_part": child.product.sku, "hash": None, "combination": None}
                    )
                except ShopProduct.DoesNotExist:
                    pass

        return variations


class VariationShopProductType(graphene.ObjectType):
    shop_product = graphene.Field(ShopProductType)
    sku_part = graphene.String()
    hash = graphene.String()
    combination = graphene.JSONString()

    def resolve_sku_part(self, info, **kwargs):
        return self.get("sku_part")

    def resolve_hash(self, info, **kwargs):
        return self.get("hash")

    def resolve_combination(self, info, **kwargs):
        return self.get("combination")

    def resolve_shop_product(self, info, **kwargs):
        return self.get("shop_product")



import factory.fuzzy as fuzzy
import graphene
from django.conf import settings

from shuup.core.models import (
    MediaFile,
    PersonContact,
    Product,
    ProductMedia,
    ProductMediaKind,
    SalesUnit,
    ShopProduct,
    ShopProductVisibility,
    Shop,
)
from shuup.testing.factories import get_default_product_type, get_default_supplier, get_default_tax_class
from shuup_graphql.front.types.product import ProductType

from ..utils.filer import filer_image_from_url


class CreateProductMutation(graphene.Mutation):
    class Arguments:
        name = graphene.String(required=True)
        business_segment = graphene.String(required=True)
        shop = graphene.Int(required=True)
        price = graphene.Decimal(required=False)
        description = graphene.String(required=False)
        image_url = graphene.String(required=False)

    ok = graphene.Boolean()
    error = graphene.String()
    product = graphene.Field(ProductType)

    def mutate(self, info, name, business_segment, shop, description=None, price=None, image_url=None):
        # request = info.context
        product = Product.objects.create(
            name=name,
            description=description,
            type=get_default_product_type(),
            tax_class=get_default_tax_class(),
            sales_unit=SalesUnit.objects.first(),
            sku=fuzzy.FuzzyText(length=10).fuzz(),
        )

        if image_url:
            path = "ProductImages/Samples/%s" % business_segment.capitalize()
            filer_image = filer_image_from_url(image_url, path)
            media_file = MediaFile.objects.create(file=filer_image)
            media_file.shops.add(shop)

            media = ProductMedia.objects.create(product=product, kind=ProductMediaKind.IMAGE, file=filer_image)
            media.save()
            media.shops.add(shop)
            product.primary_image = media
            product.save()

        # create the price and round it to the number of decimals of the currency
        shop = Shop.objects.first()
        if not price:
            price = 0
        product_price = shop.create_price(price).as_rounded()

        sp = ShopProduct.objects.create(
            product=product,
            purchasable=True,
            visibility=ShopProductVisibility.ALWAYS_VISIBLE,
            default_price_value=product_price,
            shop=shop,
            shop_primary_image=media,
        )
        sp.categories.set(shop.categories.all())
        sp.suppliers.add(get_default_supplier())
        sp.save()

        # configure prices
        if "shuup.customer_group_pricing" in settings.INSTALLED_APPS:
            from shuup.customer_group_pricing.models import CgpPrice

            CgpPrice.objects.create(
                product=product, price_value=product_price, shop=shop, group=PersonContact.get_default_group()
            )

        return CreateProductMutation(ok=True, product=product)


class ProductMutations(graphene.ObjectType):
    create_product = CreateProductMutation.Field()



# -*- coding: utf-8 -*-
from setuptools import find_packages, setup

requirements = [
    'graphene-django>=2,<3'
]

setup(
    name="shuup_graphql",
    version="0.1.2",
    author="John Yoost",
    author_email="jyoost@gmail.com",
    url="https://github.com/jyoost/shuup-graphql",
    description=("A Shuup GraphQL API implementation."),
    license="Apache 2.0",
    install_requires=requirements,
    packages=find_packages()
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os
import tempfile

BASE_DIR = os.path.dirname(__file__)

SECRET_KEY = "xyz"

INSTALLED_APPS = (
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.staticfiles",
    "easy_thumbnails",
    "filer",
    "shuup.themes.classic_gray",
    "shuup.core",
    "shuup.admin",
    "shuup.default_tax",
    "shuup.customer_group_pricing",
    "shuup.campaigns",
    "shuup.simple_supplier",
    'shuup.front',
    'shuup.front.apps.auth',
    'shuup.front.apps.customer_information',
    'shuup.front.apps.personal_order_history',
    'shuup.front.apps.saved_carts',
    'shuup.front.apps.registration',
    'shuup.front.apps.simple_order_notification',
    'shuup.front.apps.simple_search',
    "shuup.notify",
    "shuup.regions",
    "shuup.testing",
    "shuup.xtheme",

    # external apps
    "bootstrap3",
    "registration",
    'rest_framework',

    'shuup_graphql',
    'shuup_graphql_tests'
)

MIDDLEWARE_CLASSES = (
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.middleware.locale.LocaleMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.auth.middleware.SessionAuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "shuup.front.middleware.ProblemMiddleware",
    "shuup.front.middleware.ShuupFrontMiddleware",
    'shuup.core.middleware.ShuupMiddleware'
)

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": os.path.join(
            tempfile.gettempdir(),
            "shuup_graphql_tests.sqlite3"
        )
    }
}

MEDIA_ROOT = os.path.join(os.path.dirname(__file__), "var", "media")

STATIC_URL = "/static/"

ROOT_URLCONF = "shuup_graphql_tests.urls"

TIME_ZONE = 'UTC'
USE_I18N = True
USE_L10N = True
USE_TZ = True

LANGUAGE_CODE = "en"

LANGUAGES = (
    ("en", "English"),
)

PARLER_DEFAULT_LANGUAGE_CODE = "en"

PARLER_LANGUAGES = {
    None: [
        {"code": code, "name": name}
        for code, name in LANGUAGES
    ],
    "default": {
        "hide_untranslated": False
    }
}

SESSION_SERIALIZER = "django.contrib.sessions.serializers.PickleSerializer"

_TEMPLATE_CONTEXT_PROCESSORS = [
    "django.contrib.auth.context_processors.auth",
    "django.core.context_processors.debug",
    "django.core.context_processors.i18n",
    "django.core.context_processors.media",
    "django.core.context_processors.static",
    "django.core.context_processors.request",
    "django.core.context_processors.tz",
    "django.contrib.messages.context_processors.messages"
]

TEMPLATES = [
    {
        "BACKEND": "django_jinja.backend.Jinja2",
        "APP_DIRS": True,
        "OPTIONS": {
            "match_extension": ".jinja",
            "context_processors": _TEMPLATE_CONTEXT_PROCESSORS,
            "newstyle_gettext": True,
            "environment": "shuup.xtheme.engine.XthemeEnvironment",
        },
        "NAME": "jinja2",
    },
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": _TEMPLATE_CONTEXT_PROCESSORS,
            "debug": True
        }
    }
]

SHUUP_BASKET_CLASS_SPEC="shuup.core.basket.objects:Basket"
SHUUP_BASKET_ORDER_CREATOR_SPEC="shuup.core.basket.order_creator:BasketOrderCreator"
SHUUP_BASKET_STORAGE_CLASS_SPEC="shuup.core.basket.storage:DatabaseBasketStorage"



# -*- coding: utf-8 -*-
from filer.models import Image
from graphene_django import DjangoObjectType


class FilerImageType(DjangoObjectType):
    class Meta:
        model = Image
        only_fields = ("url",)



# -*- coding: utf-8 -*-
import graphene
from graphene_django import DjangoObjectType

from shuup.core.models import Manufacturer


class ManufacturerType(DjangoObjectType):
    id = graphene.Int()

    class Meta:
        model = Manufacturer



# -*- coding: utf-8 -*-
import graphene
from graphene_django import DjangoObjectType

from shuup.core.models import Category


class CategoryType(DjangoObjectType):
    id = graphene.Int()
    name = graphene.String()

    class Meta:
        model = Category



# -*- coding: utf-8 -*-
import graphene

from shuup.core.models import Manufacturer
from shuup.core.shop_provider import get_shop
from shuup_graphql.front.types.manufacturer import ManufacturerType


class ManufacturerQuery(object):
    manufactures = graphene.List(ManufacturerType, search=graphene.String())

    def resolve_manufactures(self, info, search=None, **kwargs):
        queryset = Manufacturer.objects.all()

        # if some shop is returned, then use it in the queryset
        # custom shop providers can return no shop and then
        # all manufactures will be returned, like in marketplace environments
        shop = get_shop(info.context)
        if shop:
            queryset = queryset.filter(shops=shop)

        if search:
            queryset = queryset.filter(name__icontains=search)

        return queryset
